use std::collections::HashMap;
use serde;
use serde::{Deserialize, Serialize};

pub mod client;

#[derive(Serialize, Deserialize)]
pub enum Query {
    #[serde(rename = "pull")]
    Pull {
        name: String,
        version: Option<u128>,
        tags: Vec<String>
    },
    #[serde(rename = "push")]
    Push {
        // todo: review me pls
        _data: Vec<u8>
    },
    #[serde(rename = "index")]
    Index {
        request_update: bool,
        fetch: bool
    }
}

#[derive(Serialize, Deserialize)]
pub struct Request {
    version: u32,
    id: String,
    token: Option<String>,
    query: HashMap<String, Query>
}

#[derive(Serialize, Deserialize)]
pub struct DataErrorDetails {
    actor: String,
    detailed_cause: String,
    recovery_options: Vec<String>,
}

#[derive(Serialize, Deserialize)]
pub struct DataError {
    name: String,
    cause: Option<String>,
    details: Option<DataErrorDetails>
}

#[derive(Serialize, Deserialize)]
pub enum Data {
    Pull {
        _data: Option<Vec<u8>>,
    }
}

#[derive(Serialize, Deserialize)]
pub struct Response {
    version: u32,
    id: String,
    reply_to: String,
    errors: HashMap<String, DataError>,
    data: HashMap<String, Data>
}