use std::collections::HashMap;
use std::str::FromStr;
use serde::{Deserialize, Serialize};
use url::Url;

#[derive(Clone, Serialize, Deserialize)]
pub struct Repo {
    #[serde(default)]
    pub name: String,
    pub uri: Url
}

impl Default for Repo {
    fn default() -> Self {
        Repo {
            name: String::from("Repo"),
            uri: Url::parse("tcp://0.0.0.0:0000").unwrap()
        }
    }
}

#[derive(Clone, Serialize, Deserialize)]
pub struct RepoFile {
    pub repos: HashMap<String, Repo>
}

impl RepoFile {
    pub fn from_path(path: &str) -> Result<RepoFile, String> {
        match std::fs::read_to_string(path) {
            Ok(s) => match toml::from_str(&s) {
                Ok(c) => Ok(c),
                Err(e) => Err(format!("failed to parse config: {}", e)),
            },
            Err(e) => Err(format!("failed to read config: {}", e)),
        }
    }
}