use std::collections::HashMap;
use std::io;
use std::path::Path;
use serde::{Deserialize, Serialize};
use crate::api::Request;
use crate::CONFIG;
use crate::repo::index::index_package::IndexPackage;
use crate::repo::Repo;
use crate::types::fetch::{Fetch, TryFetch};
use crate::util::create_uuid;

pub mod index_package;

#[derive(Serialize, Deserialize)]
pub struct RepoIndex {
    origin_repo: String,
    packages: HashMap<String, IndexPackage>
}

impl RepoIndex {
    // /// Fetch existing index or create a new one.
    // pub fn from_repo(repo: Repo) -> Self {
    //
    // }
    //
    // /// Create new index.
    // pub fn create_with_repo(repo: Repo) -> Self {
    // }

    /// Get repo
    pub fn get_repo(&self) -> io::Result<Repo> {
        Ok(Repo::from_name(&self.origin_repo)?)
    }

    pub fn from_path(path: &str) -> Result<RepoIndex, String> {
        match std::fs::read_to_string(path) {
            Ok(s) => match toml::from_str(&s) {
                Ok(c) => Ok(c),
                Err(e) => Err(format!("failed to parse config: {}", e)),
            },
            Err(e) => Err(format!("failed to read config: {}", e)),
        }
    }
}

impl TryFetch<Repo> for RepoIndex {
    type Error = ();
    /// Fetch
    fn try_fetch(query: Repo) -> Result<Self, Self::Error> {
        let path = CONFIG.with(|c| c.storage.index_dir.clone()) + query.uri.as_str() + ".toml";
        if Path::new(path.as_str()).exists() {
            RepoIndex::from_path(path.as_str())
                .map_err(|_| ())
        } else {
            Err(())
        }
    }
}