use std::io;
use std::io::{Error, ErrorKind};
use std::path::Path;
use url::Url;
use crate::CONFIG;
use crate::config::repos::RepoFile;

/// Indexed repos
pub mod index;

pub struct Repo {
    name: String,
    uri: Url,
}

impl Repo {
    pub fn from_name(name: &String) -> io::Result<Repo> {
        let r = RepoFile::from_path(CONFIG.with(|c| c.storage.repo_file.clone()).as_str())
            .map_err(|e| Error::new(ErrorKind::Other, e))?;
        let r = r
            .repos
            .get(name)
            .ok_or(Error::new(ErrorKind::InvalidData, "Could not get repo"))?;
        Ok(Repo {
            name: r.name.clone(),
            uri: r.uri.clone()
        })
    }

    pub fn get_name(&self) -> String {
        self.name.clone()
    }

    pub fn get_uri(&self) -> Url {
        self.uri.clone()
    }

    /// Fetch indexed repo
    pub fn get_index(&self) {

    }
}